@@ -18,7 +18,13 @@
  * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
- */
+ *
+ * $Id: add.c,v 1.12 1999/07/24 01:41:52 lister Exp $
+*/
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdio.h>
 #include <Xm/Form.h>
@@ -26,7 +32,7 @@
 #include <Xm/PushB.h>
 #include <Xm/Scale.h>
 #include <Xm/MessageB.h>
-#include "../../src/Matrix.h"
+#include <Xbae/Matrix.h>
 
 /*
  * Add multiple rows or columns at a time.
@@ -36,22 +42,28 @@
  */
 
 void LoadMatrix();
-void RowOrientationCB(), ColumnOrientationCB(), AddCB(), SetNumberCB();
+void RowOrientationCB();
+void ColumnOrientationCB();
+void AddCB();
+void DelCB();
+void SetNumberCB();
 void SetPositionCB();
 void defaultCB();
 void enterCB();
-void resizeCB();
 
 typedef enum {
     RowOrientation,
     ColumnOrientation
 } Orientation;
+
 typedef struct {
     Widget matrix;
     Orientation orientation;
     int number;
     int row, column;
     int num_rows, num_columns;
+    Dimension fixed_rows, fixed_columns;
+    Dimension trailing_fixed_rows, trailing_fixed_columns;    
 } AddDataRec, *AddData;
 
 int
@@ -59,10 +71,10 @@ main(argc, argv)
 int argc;
 char *argv[];
 {
-    Widget toplevel, form, rc, menu, row, column, option, number, add;
+    Widget toplevel, form, rc, menu, row, column, option, number, add, del;
     AddDataRec addData;
     XtAppContext app;
-    Arg args[2];
+    Arg args[8];
     int n;
 #ifdef USE_EDITRES
     extern void _XEditResCheckMessages();
@@ -81,11 +93,11 @@ char *argv[];
      * Create a Form to hold everything
      */
     form = XtCreateManagedWidget("form",
-                                 xmFormWidgetClass,     toplevel,
+                                 xmFormWidgetClass, toplevel,
                                  NULL, 0);
 
     rc = XtCreateManagedWidget("rc",
-			       xmRowColumnWidgetClass,     form,
+			       xmRowColumnWidgetClass, form,
 			       NULL, 0);
 
     /*
@@ -97,7 +109,7 @@ char *argv[];
      * Create a menu button to select row orientation
      */
     row = XtVaCreateManagedWidget("row",
-				  xmPushButtonWidgetClass,	menu,
+				  xmPushButtonWidgetClass, menu,
 				  NULL);
     XtAddCallback(row, XmNactivateCallback, RowOrientationCB,
 		  (XtPointer)&addData);
@@ -106,7 +118,7 @@ char *argv[];
      * Create a menu button to select column orientation
      */
     column = XtVaCreateManagedWidget("column",
-				     xmPushButtonWidgetClass,   menu,
+				     xmPushButtonWidgetClass, menu,
 				     NULL);
     XtAddCallback(column, XmNactivateCallback, ColumnOrientationCB,
 		  (XtPointer)&addData);
@@ -115,8 +127,8 @@ char *argv[];
      * Setup and create the option menu
      */
     n = 0;
-    XtSetArg(args[n], XmNsubMenuId, menu);		n++;
-    XtSetArg(args[n], XmNmenuHistory, row);		n++;
+    XtSetArg(args[n], XmNsubMenuId, menu); n++;
+    XtSetArg(args[n], XmNmenuHistory, row); n++;
     option = XmCreateOptionMenu(rc, "option", args, n);
     XtManageChild(option);
 
@@ -125,14 +137,15 @@ char *argv[];
     /*
      * Create a scale to determine how many to add
      */
-    number = XtVaCreateManagedWidget("number",
-				     xmScaleWidgetClass,rc,
-				     XmNvalue,		1,
-				     XmNleftAttachment,	XmATTACH_WIDGET,
-				     XmNleftWidget,	option,
-				     XmNbottomAttachment,XmATTACH_OPPOSITE_WIDGET,
-				     XmNbottomWidget,	option,
-				     NULL);
+    number = XtVaCreateManagedWidget(
+	"number", xmScaleWidgetClass, rc,
+	XmNvalue, 1,
+	XmNleftAttachment, XmATTACH_WIDGET,
+	XmNleftWidget, option,
+	XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET,
+	XmNbottomWidget, option,
+	NULL);
+
     XtAddCallback(number, XmNvalueChangedCallback, SetNumberCB,
 		  (XtPointer)&addData);
     addData.number = 1;
@@ -140,42 +153,57 @@ char *argv[];
     /*
      * Create a button to do the adding
      */
-    add = XtVaCreateManagedWidget("add",
-				  xmPushButtonWidgetClass,rc,
-				  XmNleftAttachment,	XmATTACH_WIDGET,
-				  XmNleftWidget,	number,
-				  XmNbottomAttachment,XmATTACH_OPPOSITE_WIDGET,
-				  XmNbottomWidget,	number,
-				  NULL);
+    add = XtVaCreateManagedWidget(
+	"add", xmPushButtonWidgetClass, rc,
+	XmNleftAttachment, XmATTACH_WIDGET,
+	XmNleftWidget, number,
+	XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET,
+	XmNbottomWidget, number,
+	NULL);
+
     XtAddCallback(add, XmNactivateCallback, AddCB, (XtPointer)&addData);
 
+    del = XtVaCreateManagedWidget(
+	"Delete", xmPushButtonWidgetClass, rc,
+	XmNleftAttachment, XmATTACH_WIDGET,
+	XmNleftWidget, add,
+	XmNbottomAttachment,XmATTACH_OPPOSITE_WIDGET,
+	XmNbottomWidget, add,
+	NULL);
+
+    XtAddCallback(del, XmNactivateCallback, DelCB, (XtPointer)&addData);
+
     /*
      * Create a Matrix widget.
      */
-    addData.matrix =
-	XtVaCreateManagedWidget("mw",
-				xbaeMatrixWidgetClass,	form,
-				XmNleftAttachment,	XmATTACH_FORM,
-				XmNrightAttachment,	XmATTACH_FORM,
-				XmNbottomAttachment,	XmATTACH_FORM,
-		  		XmNtopAttachment,	XmATTACH_WIDGET,
-		  		XmNtopWidget,		rc,
-				NULL);
+
+    addData.matrix = XtVaCreateManagedWidget(
+	"mw", xbaeMatrixWidgetClass, form,
+	XmNleftAttachment, XmATTACH_FORM,
+	XmNrightAttachment, XmATTACH_FORM,
+	XmNbottomAttachment, XmATTACH_FORM,
+	XmNtopAttachment, XmATTACH_WIDGET,
+	XmNtopWidget, rc,
+	NULL);
+
     XtAddCallback(addData.matrix, XmNselectCellCallback, SetPositionCB,
 		  (XtPointer)&addData);
     XtAddCallback(addData.matrix, XmNdefaultActionCallback, defaultCB,
 		  (XtPointer)&addData);
     XtAddCallback(addData.matrix, XmNenterCellCallback, enterCB,
 		  (XtPointer)&addData);
-    XtAddCallback(addData.matrix, XmNresizeColumnCallback, resizeCB,
-		  (XtPointer)&addData);
+
     XtVaGetValues(addData.matrix,
 		  XmNrows, &addData.num_rows,
 		  XmNcolumns, &addData.num_columns,
+		  XmNfixedRows, &addData.fixed_rows,
+		  XmNfixedColumns, &addData.fixed_columns,
+		  XmNtrailingFixedRows, &addData.trailing_fixed_rows,
+		  XmNtrailingFixedColumns, &addData.trailing_fixed_columns,
 		  NULL);
+
     addData.row = 0;
     addData.column = 0;
-
     /*
      * Load the matrix with default values
      */
@@ -207,7 +235,8 @@ Widget w;
 AddData addData;
 XtPointer call_data;
 {
-    if (addData->orientation != ColumnOrientation) {
+    if (addData->orientation != ColumnOrientation)
+    {
 	addData->orientation = ColumnOrientation;
 	XbaeMatrixDeselectRow(addData->matrix, addData->row);
     }
@@ -220,33 +249,25 @@ Widget w;
 AddData addData;
 XbaeMatrixEnterCellCallbackStruct *cbs;
 {
+#if 0
+    int rows, cols;
+    char str[16];
+#endif
+
     if( cbs->row == 1 || cbs->row == 4 || cbs->column == 1 )
+    {
 	cbs->doit = True;
+#if 0
+	XtVaGetValues(w, XmNrows, &rows, XmNcolumns, &cols, NULL);
+	(void)sprintf(str, "%d", rand() % 16381);
+	XbaeMatrixSetCell(w, rows - 1, cols / 2, str);
+	printf("set cell to %s\n", str);
+#endif
+    }
     else
 	cbs->doit = False;
 }
 
-/* ARGSUSED */
-void
-resizeCB(w, addData, cbs)
-Widget w;
-AddData addData;
-XbaeMatrixResizeColumnCallbackStruct *cbs;
-{
-    int i;
-    int width = cbs->column_widths[ cbs->which ];
-
-    if( width > 30 )
-	return;
-    
-    if( width > 15 )		/* Adjust both adjacent columns */
-	cbs->column_widths[ cbs->which + 1 ] = 30 - width;
-
-    if( width <= 15 )		/* Shrink all columns accordingly */
-	for( i = 0; i < cbs->columns; i++ )
-	    cbs->column_widths[ i ] = width;
-}
-
 /* ARGSUSED */
 void
 defaultCB(w, addData, cbs)
@@ -306,8 +327,8 @@ XtPointer call_data;
 		rows[i * addData->num_columns + j] = XtNewString(buf);
 	    }
 
-	XbaeMatrixAddRows(addData->matrix, addData->row + 1, rows, NULL, NULL,
-			  addData->number);
+	XbaeMatrixAddRows(addData->matrix, addData->num_rows - 1,
+			  rows, NULL, NULL, addData->number);
 
 	length = addData->num_columns * addData->number;
 	for (i = 0; i < length; i++)
@@ -330,8 +351,8 @@ XtPointer call_data;
 		columns[i * addData->number + j] = XtNewString(buf);
 	    }
 
-	XbaeMatrixAddColumns(addData->matrix, addData->column + 1, columns,
-			     NULL, widths, NULL, NULL, NULL, NULL,
+	XbaeMatrixAddColumns(addData->matrix, addData->num_columns - 1,
+			     columns, NULL, widths, NULL, NULL, NULL, NULL,
 			     addData->number);
 
 	XtFree((XtPointer)widths);
@@ -345,6 +366,43 @@ XtPointer call_data;
     }
 }
 
+/* ARGSUSED */
+void
+DelCB(w, addData, call_data)
+Widget w;
+AddData addData;
+XtPointer call_data;
+{
+    /*
+     * Delete rows or columns
+     */
+
+    switch (addData->orientation) {
+
+    case RowOrientation:
+	if( addData->num_rows - addData->fixed_rows -
+	    addData->trailing_fixed_rows - addData->number <= 0 )
+	    return;
+
+	XbaeMatrixDeleteRows(addData->matrix, addData->num_rows -
+			     addData->number, addData->number);
+
+	addData->num_rows -= addData->number;
+	break;
+
+    case ColumnOrientation:
+	if( addData->num_columns - addData->fixed_columns -
+	    addData->trailing_fixed_columns - addData->number <= 0 )
+	    return;
+    
+	XbaeMatrixDeleteColumns(addData->matrix, addData->num_columns -
+				addData->number, addData->number);
+
+	addData->num_columns -= addData->number;
+	break;
+    }
+}
+
 /* ARGSUSED */
 void
 SetNumberCB(w, addData, call_data)
@@ -364,16 +422,16 @@ XbaeMatrixSelectCellCallbackStruct *call_data;
 {
     switch (addData->orientation) {
     case RowOrientation:
-	XbaeMatrixDeselectRow(addData->matrix, addData->row);
+	XbaeMatrixUnhighlightRow(addData->matrix, addData->row);
 	addData->row = call_data->row;
 	addData->column = call_data->column;
-	XbaeMatrixSelectRow(addData->matrix, addData->row);
+	XbaeMatrixHighlightRow(addData->matrix, addData->row);
 	break;
     case ColumnOrientation:
-	XbaeMatrixDeselectColumn(addData->matrix, addData->column);
+	XbaeMatrixUnhighlightColumn(addData->matrix, addData->column);
 	addData->row = call_data->row;
 	addData->column = call_data->column;
-	XbaeMatrixSelectColumn(addData->matrix, addData->column);
+	XbaeMatrixHighlightColumn(addData->matrix, addData->column);
 	break;
     }
 
