@@ -1,7 +1,13 @@
 /*
- * AUTHOR: Jay Schmidgall <jay@spdbump.sungardss.com>
+ * AUTHOR: Jay Schmidgall <jay.schmidgall@spdbump.sungardss.com>
+ *
+ * $Id: choice.c,v 1.9 1999/07/24 01:49:10 lister Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <Xbae/Matrix.h>
 #include <Xbae/Actions.h>
 #include <Xm/Form.h>
@@ -66,6 +72,286 @@ typedef struct {
     int	    value;
 } SetValueStruct;
 
+typedef struct {
+    int value;
+    Pixel armColor;
+    Pixel foreColor;
+    Pixel backColor;
+} SelectModeStruct;
+
+enum { SelectSelect, SelectHighlight, SelectArm };
+
+void
+cbSelectMode(w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+    SelectModeStruct *sms;
+    XmToggleButtonCallbackStruct *cbs = (XmToggleButtonCallbackStruct *) call;
+    SetValueStruct *svs = (SetValueStruct *) client;
+    unsigned char *rowShadows = NULL;
+    unsigned char *columnShadows = NULL;
+    unsigned char **cellShadows = NULL;
+    unsigned char cellShadowType;
+    Pixel **backgrounds = NULL;
+    int rows, columns, r, c;
+	
+    XtVaGetValues(svs->mw,
+		  XmNuserData, &sms,
+		  XmNrows, &rows,
+		  XmNcolumns, &columns,
+		  XmNcellShadowType, &cellShadowType,
+		  NULL);
+    
+    if (cbs->set)
+	sms->value = svs->value;
+
+    if (sms->value == SelectArm)
+    {
+	cellShadows = (unsigned char **) XtMalloc(rows * sizeof(unsigned char *));
+	backgrounds = (Pixel **) XtMalloc(rows * sizeof(Pixel *));
+	rowShadows = (unsigned char*) XtMalloc(rows * sizeof(unsigned char));
+	columnShadows = (unsigned char*) XtMalloc(columns * sizeof(unsigned char));
+	for (r = 0; r < rows; r++)
+	{
+	    cellShadows[r] = (unsigned char*) XtMalloc(columns * sizeof(unsigned char));
+	    backgrounds[r] = (Pixel *) XtMalloc(columns * sizeof(Pixel));
+	    rowShadows[r] = cellShadowType;
+	    for (c = 0; c < columns; c++)
+	    {
+		cellShadows[r][c] = cellShadowType;
+		backgrounds[r][c] = sms->backColor;
+		columnShadows[c] = cellShadowType;
+	    }
+	}
+    }
+    
+    XtVaSetValues(svs->mw,
+		  XmNcellBackgrounds, backgrounds,
+		  XmNcellShadowTypes, cellShadows,
+		  XmNrowShadowTypes, rowShadows,
+		  XmNcolumnShadowTypes, columnShadows,
+		  NULL);
+
+    XbaeMatrixRefresh(svs->mw);
+}
+
+
+void
+UnarmAll(w, sms)
+Widget w;
+SelectModeStruct *sms;
+{
+    int rows, columns, r, c;
+    unsigned char *rowShadows, *columnShadows, **cellShadows;
+    unsigned char grid, cellShadowType;
+    Pixel **backgrounds;
+
+    if (sms->value != SelectArm)
+	return;
+    
+    XtVaGetValues(w,
+		  XmNrows, &rows,
+		  XmNcolumns, &columns,
+		  XmNgridType, &grid,
+		  XmNcellBackgrounds, &backgrounds,
+		  XmNcellShadowType, &cellShadowType,
+		  XmNcellShadowTypes, &cellShadows,
+		  XmNrowShadowTypes, &rowShadows,
+		  XmNcolumnShadowTypes, &columnShadows,
+		  NULL);
+
+    for (r = 0; r < rows; r++)
+    {
+	rowShadows[r] = cellShadowType;
+	for (c = 0; c < columns; c++)
+	{
+	    cellShadows[r][c] = cellShadowType;
+	    backgrounds[r][c] = sms->backColor;
+	    columnShadows[c] = cellShadowType;
+	    /* Ooops, internal function! */
+	    XbaeMatrixRefreshCell(w, r, c);
+	}
+    }
+}
+
+void
+ArmColumn(w, column, sms)
+Widget w;
+int column;
+SelectModeStruct *sms;
+{
+    int rows, columns, r;
+    unsigned char *rowShadows, *columnShadows, **cellShadows;
+    unsigned char grid, cellShadowType;
+    Pixel **backgrounds;
+    
+    XtVaGetValues(w,
+		  XmNrows, &rows,
+		  XmNcolumns, &columns,
+		  XmNgridType, &grid,
+		  XmNcellBackgrounds, &backgrounds,
+		  XmNcellShadowType, &cellShadowType,
+		  XmNcellShadowTypes, &cellShadows,
+		  XmNrowShadowTypes, &rowShadows,
+		  XmNcolumnShadowTypes, &columnShadows,
+		  NULL);
+
+    if (grid == XmGRID_COLUMN_SHADOW)
+	switch (columnShadows[column])
+	{
+	    case XmSHADOW_IN:
+		columnShadows[column] = XmSHADOW_OUT;
+		break;
+
+	    case XmSHADOW_OUT:
+		columnShadows[column] = XmSHADOW_IN;
+		break;
+
+	    case XmSHADOW_ETCHED_IN:
+		columnShadows[column] = XmSHADOW_ETCHED_OUT;
+		break;
+
+	    case XmSHADOW_ETCHED_OUT:
+		columnShadows[column] = XmSHADOW_ETCHED_IN;
+		break;
+	}
+	    
+    for (r = 0; r < rows; r++)
+    {
+	backgrounds[r][column] = sms->armColor;
+
+	/* Ooops, internal function! */
+	XbaeMatrixRefreshCell(w, r, column);
+    }
+}
+
+void
+ArmRow(w, row, sms)
+Widget w;
+int row;
+SelectModeStruct *sms;
+{
+    int rows, columns, c;
+    unsigned char *rowShadows, *columnShadows, **cellShadows;
+    unsigned char grid, cellShadowType;
+    Pixel **backgrounds;
+    
+    XtVaGetValues(w,
+		  XmNrows, &rows,
+		  XmNcolumns, &columns,
+		  XmNgridType, &grid,
+		  XmNcellBackgrounds, &backgrounds,
+		  XmNcellShadowType, &cellShadowType,
+		  XmNcellShadowTypes, &cellShadows,
+		  XmNrowShadowTypes, &rowShadows,
+		  XmNcolumnShadowTypes, &columnShadows,
+		  NULL);
+
+    if (grid == XmGRID_ROW_SHADOW)
+	switch (rowShadows[row])
+	{
+	    case XmSHADOW_IN:
+		rowShadows[row] = XmSHADOW_OUT;
+		break;
+
+	    case XmSHADOW_OUT:
+		rowShadows[row] = XmSHADOW_IN;
+		break;
+
+	    case XmSHADOW_ETCHED_IN:
+		rowShadows[row] = XmSHADOW_ETCHED_OUT;
+		break;
+
+	    case XmSHADOW_ETCHED_OUT:
+		rowShadows[row] = XmSHADOW_ETCHED_IN;
+		break;
+	}
+	    
+    for (c = 0; c < columns; c++)
+    {
+	backgrounds[row][c] = sms->armColor;
+	
+	/* Ooops, internal function! */
+	XbaeMatrixRefreshCell(w, row, c);
+    }
+}
+
+void
+ArmCell(w, row, column, sms)
+Widget w;
+int row;
+int column;
+SelectModeStruct *sms;
+{
+    int rows, columns, r, c;
+    unsigned char *rowShadows, *columnShadows, **cellShadows;
+    unsigned char grid, cellShadowType;
+    Pixel **backgrounds;
+
+    XtVaGetValues(w,
+		  XmNrows, &rows,
+		  XmNcolumns, &columns,
+		  XmNgridType, &grid,
+		  XmNcellBackgrounds, &backgrounds,
+		  XmNcellShadowType, &cellShadowType,
+		  XmNcellShadowTypes, &cellShadows,
+		  XmNrowShadowTypes, &rowShadows,
+		  XmNcolumnShadowTypes, &columnShadows,
+		  NULL);
+
+    for (r = 0; r < rows; r++)
+    {
+	backgrounds[r][column] = sms->armColor;
+	switch (cellShadows[r][column])
+	{
+	    case XmSHADOW_IN:
+		cellShadows[r][column] = XmSHADOW_OUT;
+		break;
+
+	    case XmSHADOW_OUT:
+		cellShadows[r][column] = XmSHADOW_IN;
+		break;
+
+	    case XmSHADOW_ETCHED_IN:
+		cellShadows[r][column] = XmSHADOW_ETCHED_OUT;
+		break;
+
+	    case XmSHADOW_ETCHED_OUT:
+		cellShadows[r][column] = XmSHADOW_ETCHED_IN;
+		break;
+	}
+	/* Ooops, internal function! */
+	XbaeMatrixRefreshCell(w, r, column);
+    }
+
+    for (c = 0; c < columns; c++)
+	if (c != column)
+	{
+	    backgrounds[row][c] = sms->armColor;
+	    switch (cellShadows[row][c])
+	    {
+		case XmSHADOW_IN:
+		    cellShadows[row][c] = XmSHADOW_OUT;
+		    break;
+
+		case XmSHADOW_OUT:
+		    cellShadows[row][c] = XmSHADOW_IN;
+		    break;
+
+		case XmSHADOW_ETCHED_IN:
+		    cellShadows[row][c] = XmSHADOW_ETCHED_OUT;
+		    break;
+
+		case XmSHADOW_ETCHED_OUT:
+		    cellShadows[row][c] = XmSHADOW_ETCHED_IN;
+		    break;
+	    }
+	    /* Ooops, internal function! */
+	    XbaeMatrixRefreshCell(w, row, c);
+	}
+}
 
 void
 cbRadio(w, client, call)
@@ -100,28 +386,32 @@ char *name;
 {
     Widget frame, frame2, label, rc;
     
-    frame = XtVaCreateManagedWidget("frame", xmFrameWidgetClass, parent,
-				    NULL);
+    frame = XtVaCreateManagedWidget(
+	"frame", xmFrameWidgetClass, parent,
+	NULL);
 
-    frame2 = XtVaCreateManagedWidget("frame", xmFrameWidgetClass, frame,
-				     XmNchildType,		    XmFRAME_TITLE_CHILD,
-				     XmNchildHorizontalAlignment,   XmALIGNMENT_CENTER,
-				     XmNchildVerticalAlignment,	    XmALIGNMENT_CENTER,
-				     XmNmarginWidth,		    4,
-				     NULL);
-
-    label = XtVaCreateManagedWidget(name, xmLabelWidgetClass, frame2,
-				    NULL);
+    frame2 = XtVaCreateManagedWidget(
+	"frame", xmFrameWidgetClass, frame,
+	XmNchildType,		    XmFRAME_TITLE_CHILD,
+	XmNchildHorizontalAlignment,   XmALIGNMENT_CENTER,
+	XmNchildVerticalAlignment,	    XmALIGNMENT_CENTER,
+	XmNmarginWidth,		    4,
+	NULL);
     
-    rc = XtVaCreateManagedWidget("rc", xmRowColumnWidgetClass, frame,
-				 XmNchildType,	    XmFRAME_WORKAREA_CHILD,
-				 XmNadjustLast,	    False,
-				 XmNradioBehavior,  True,
-				 XmNradioAlwaysOne, True,
-				 XmNorientation,    XmHORIZONTAL,
-				 XmNnumColumns,	    3,
-				 XmNpacking,	    XmPACK_COLUMN,
-				 NULL);
+    label = XtVaCreateManagedWidget(
+	name, xmLabelWidgetClass, frame2,
+	NULL);
+    
+    rc = XtVaCreateManagedWidget(
+	"rc", xmRowColumnWidgetClass, frame,
+	XmNchildType,	    XmFRAME_WORKAREA_CHILD,
+	XmNadjustLast,	    False,
+	XmNradioBehavior,   True,
+	XmNradioAlwaysOne,  True,
+	XmNnumColumns,	    1,
+	XmNorientation,	    XmVERTICAL,
+	XmNpacking,	    XmPACK_TIGHT,
+	NULL);
     return rc;
 }
 
@@ -134,27 +424,31 @@ int min, max;
 {
     Widget frame, frame2, label, scale;
 
-    frame = XtVaCreateManagedWidget("frame", xmFrameWidgetClass, parent,
+    frame = XtVaCreateManagedWidget(
+	"frame", xmFrameWidgetClass, parent,
 				    NULL);
 
-    frame2 = XtVaCreateManagedWidget("frame", xmFrameWidgetClass, frame,
-				     XmNchildType,		    XmFRAME_TITLE_CHILD,
-				     XmNchildHorizontalAlignment,   XmALIGNMENT_CENTER,
-				     XmNchildVerticalAlignment,	    XmALIGNMENT_CENTER,
-				     XmNmarginWidth,		    4,
-				     NULL);
+    frame2 = XtVaCreateManagedWidget(
+	"frame", xmFrameWidgetClass, frame,
+	XmNchildType,		    XmFRAME_TITLE_CHILD,
+	XmNchildHorizontalAlignment,   XmALIGNMENT_CENTER,
+	XmNchildVerticalAlignment,	    XmALIGNMENT_CENTER,
+	XmNmarginWidth,		    4,
+	NULL);
 
-    label = XtVaCreateManagedWidget(name, xmLabelWidgetClass, frame2,
-				    NULL);
+    label = XtVaCreateManagedWidget(
+	name, xmLabelWidgetClass, frame2,
+	NULL);
     
-    scale = XtVaCreateManagedWidget("scale", xmScaleWidgetClass, frame,
-				    XmNchildType,   XmFRAME_WORKAREA_CHILD,
-				    XmNorientation, XmHORIZONTAL,
-				    XmNminimum,	    min,
-				    XmNmaximum,	    max,
-				    XmNscaleMultiple, 1,
-				    XmNshowValue,   True,
-				    NULL);
+    scale = XtVaCreateManagedWidget(
+	"scale", xmScaleWidgetClass, frame,
+	XmNchildType,   XmFRAME_WORKAREA_CHILD,
+	XmNorientation, XmHORIZONTAL,
+	XmNminimum,	    min,
+	XmNmaximum,	    max,
+	XmNscaleMultiple, 1,
+	XmNshowValue,   True,
+	NULL);
     return scale;
 }
 
@@ -169,56 +463,76 @@ Widget mw;
     String resource = XtNewString(XmNgridType);
 
     rc = createButtonBox(parent, "Grid Type");
+    XtVaSetValues(rc,
+		  XmNorientation,   XmHORIZONTAL,
+		  XmNnumColumns,    3,
+		  XmNpacking,	    XmPACK_COLUMN,
+		  NULL);
 
-    button = XtVaCreateManagedWidget("XmGRID_NONE", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmGRID_CELL_LINE", xmToggleButtonWidgetClass, rc,
+	XmNset, True,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
-    svs->value = XmGRID_NONE;
+    svs->value = XmGRID_CELL_LINE;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmGRID_LINE", xmToggleButtonWidgetClass, rc,
-				     XmNset, True,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmGRID_ROW_LINE", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
-    svs->value = XmGRID_LINE;
+    svs->value = XmGRID_ROW_LINE;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmGRID_SHADOW_IN", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmGRID_COLUMN_LINE", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
-    svs->value = XmGRID_SHADOW_IN;
+    svs->value = XmGRID_COLUMN_LINE;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmGRID_SHADOW_OUT", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmGRID_CELL_SHADOW", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
-    svs->value = XmGRID_SHADOW_OUT;
+    svs->value = XmGRID_CELL_SHADOW;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmGRID_ROW_SHADOW", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmGRID_ROW_SHADOW", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmGRID_ROW_SHADOW;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmGRID_COLUMN_SHADOW", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmGRID_COLUMN_SHADOW", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmGRID_COLUMN_SHADOW;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
-}
+    
+    button = XtVaCreateManagedWidget(
+	"XmGRID_NONE", xmToggleButtonWidgetClass, rc,
+	NULL);
+    svs = XtNew(SetValueStruct);
+    svs->mw = mw;
+    svs->resource = resource;
+    svs->value = XmGRID_NONE;
+    XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
+    }
 
 
 void
@@ -231,39 +545,38 @@ Widget mw;
     String resource = XtNewString(XmNcellShadowType);
 
     rc = createButtonBox(parent, "Cell Shadow Type");
-    XtVaSetValues(rc,
-		  XmNnumColumns, 1,
-		  XmNorientation, XmVERTICAL,
-		  XmNpacking, XmPACK_TIGHT,
-		  NULL);
 
-    button = XtVaCreateManagedWidget("XmSHADOW_IN", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmSHADOW_IN", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmSHADOW_IN;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmSHADOW_OUT", xmToggleButtonWidgetClass, rc,
-				     XmNset, True,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmSHADOW_OUT", xmToggleButtonWidgetClass, rc,
+	XmNset, True,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmSHADOW_OUT;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmSHADOW_ETCHED_IN", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmSHADOW_ETCHED_IN", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmSHADOW_ETCHED_IN;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmSHADOW_ETCHED_OUT", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmSHADOW_ETCHED_OUT", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
@@ -282,40 +595,38 @@ Widget mw;
     String resource = XtNewString(XmNshadowType);
 
     rc = createButtonBox(parent, "Shadow Type");
-    XtVaSetValues(rc,
-		  XmNnumColumns, 1,
-		  XmNorientation, XmVERTICAL,
-		  XmNpacking, XmPACK_TIGHT,
-		  NULL);
 
-
-    button = XtVaCreateManagedWidget("XmSHADOW_IN", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmSHADOW_IN", xmToggleButtonWidgetClass, rc,
+	XmNset, True,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmSHADOW_IN;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmSHADOW_OUT", xmToggleButtonWidgetClass, rc,
-				     XmNset, True,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmSHADOW_OUT", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmSHADOW_OUT;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmSHADOW_ETCHED_IN", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmSHADOW_ETCHED_IN", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmSHADOW_ETCHED_IN;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmSHADOW_ETCHED_OUT", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmSHADOW_ETCHED_OUT", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
@@ -335,33 +646,37 @@ Widget mw;
 
     rc = createButtonBox(parent, "Scrollbar Placement");
 
-    button = XtVaCreateManagedWidget("XmTOP_LEFT", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmTOP_LEFT", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmTOP_LEFT;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
 
-    button = XtVaCreateManagedWidget("XmTOP_RIGHT", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmTOP_RIGHT", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmTOP_RIGHT;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
 
-    button = XtVaCreateManagedWidget("XmBOTTOM_LEFT", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmBOTTOM_LEFT", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmBOTTOM_LEFT;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
 
-    button = XtVaCreateManagedWidget("XmBOTTOM_RIGHT", xmToggleButtonWidgetClass, rc,
-				     XmNset, True,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmBOTTOM_RIGHT", xmToggleButtonWidgetClass, rc,
+	XmNset, True,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
@@ -380,19 +695,25 @@ Widget mw;
     String resource = XtNewString(XmNfill);
 
     rc = createButtonBox(parent, "Fill");
-    XtVaSetValues(rc, XmNnumColumns, 1, NULL);
+    XtVaSetValues(rc,
+		  XmNorientation,   XmHORIZONTAL,
+		  XmNnumColumns,    1,
+		  XmNpacking,	    XmPACK_COLUMN,
+		  NULL);
 
-    button = XtVaCreateManagedWidget("True", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"True", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = True;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("False", xmToggleButtonWidgetClass, rc,
-				     XmNset, True,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"False", xmToggleButtonWidgetClass, rc,
+	XmNset, True,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
@@ -411,19 +732,25 @@ Widget mw;
     String resource = XtNewString(XmNallowColumnResize);
 
     rc = createButtonBox(parent, "Allow Column Resize");
-    XtVaSetValues(rc, XmNnumColumns, 1, NULL);
+    XtVaSetValues(rc,
+		  XmNorientation,   XmHORIZONTAL,
+		  XmNnumColumns,    1,
+		  XmNpacking,	    XmPACK_COLUMN,
+		  NULL);
 
-    button = XtVaCreateManagedWidget("True", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"True", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = True;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("False", xmToggleButtonWidgetClass, rc,
-				     XmNset, True,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"False", xmToggleButtonWidgetClass, rc,
+	XmNset, True,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
@@ -431,6 +758,118 @@ Widget mw;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
 }
     
+void
+createButtonLabels(parent, mw)
+Widget parent;
+Widget mw;
+{
+    Widget rc, button;
+    SetValueStruct *svs;
+    String resource = XtNewString(XmNbuttonLabels);
+
+    rc = createButtonBox(parent, "Button Labels");
+    XtVaSetValues(rc,
+		  XmNorientation,   XmHORIZONTAL,
+		  XmNnumColumns,    1,
+		  XmNpacking,	    XmPACK_COLUMN,
+		  NULL);
+
+    button = XtVaCreateManagedWidget(
+	"True", xmToggleButtonWidgetClass, rc,
+	NULL);
+    svs = XtNew(SetValueStruct);
+    svs->mw = mw;
+    svs->resource = resource;
+    svs->value = True;
+    XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
+    
+    button = XtVaCreateManagedWidget(
+	"False", xmToggleButtonWidgetClass, rc,
+	XmNset, True,
+	NULL);
+    svs = XtNew(SetValueStruct);
+    svs->mw = mw;
+    svs->resource = resource;
+    svs->value = False;
+    XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
+}
+
+void
+createRowAttached(parent, mw)
+Widget parent;
+Widget mw;
+{
+    Widget rc, button;
+    SetValueStruct *svs;
+    String resource = XtNewString(XmNtrailingAttachedBottom);
+
+    rc = createButtonBox(parent, "Trailing Attached Bottom");
+    XtVaSetValues(rc,
+		  XmNorientation,   XmHORIZONTAL,
+		  XmNnumColumns,    1,
+		  XmNpacking,	    XmPACK_COLUMN,
+		  NULL);
+
+    button = XtVaCreateManagedWidget(
+	"True", xmToggleButtonWidgetClass, rc,
+	NULL);
+    svs = XtNew(SetValueStruct);
+    svs->mw = mw;
+    svs->resource = resource;
+    svs->value = True;
+    XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
+    
+    button = XtVaCreateManagedWidget(
+	"False", xmToggleButtonWidgetClass, rc,
+	XmNset, True,
+	NULL);
+    svs = XtNew(SetValueStruct);
+    svs->mw = mw;
+    svs->resource = resource;
+    svs->value = False;
+    XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
+}
+
+void
+createSelectMode(parent, mw)
+Widget parent;
+Widget mw;
+{
+    Widget rc, button;
+    SetValueStruct *svs;
+
+    rc = createButtonBox(parent, "Select Mode (not a resource)");
+    XtVaSetValues(rc,
+		  XmNorientation,   XmHORIZONTAL,
+		  XmNnumColumns,    1,
+		  XmNpacking,	    XmPACK_COLUMN,
+		  NULL);
+
+    button = XtVaCreateManagedWidget(
+	"Select", xmToggleButtonWidgetClass, rc,
+	XmNset, True,
+	NULL);
+    svs = XtNew(SetValueStruct);
+    svs->mw = mw;
+    svs->value = SelectSelect;
+    XtAddCallback(button, XmNvalueChangedCallback, cbSelectMode, (XtPointer) svs);
+    
+    button = XtVaCreateManagedWidget(
+	"Highlight", xmToggleButtonWidgetClass, rc,
+	NULL);
+    svs = XtNew(SetValueStruct);
+    svs->mw = mw;
+    svs->value = SelectHighlight;
+    XtAddCallback(button, XmNvalueChangedCallback, cbSelectMode, (XtPointer) svs);
+    
+    button = XtVaCreateManagedWidget(
+	"Arm", xmToggleButtonWidgetClass, rc,
+	NULL);
+    svs = XtNew(SetValueStruct);
+    svs->mw = mw;
+    svs->value = SelectArm;
+    XtAddCallback(button, XmNvalueChangedCallback, cbSelectMode, (XtPointer) svs);
+}
 
 void
 createVerticalDisplay(parent, mw)
@@ -442,31 +881,29 @@ Widget mw;
     String resource = XtNewString(XmNverticalScrollBarDisplayPolicy);
 
     rc = createButtonBox(parent, "Vertical Display Policy");
-    XtVaSetValues(rc,
-		  XmNnumColumns, 1,
-		  XmNorientation, XmVERTICAL,
-		  XmNpacking, XmPACK_TIGHT,
-		  NULL);
 
-    button = XtVaCreateManagedWidget("XmDISPLAY_NONE", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmDISPLAY_NONE", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmDISPLAY_NONE;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmDISPLAY_AS_NEEDED", xmToggleButtonWidgetClass, rc,
-				     XmNset, True,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmDISPLAY_AS_NEEDED", xmToggleButtonWidgetClass, rc,
+	XmNset, True,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmDISPLAY_AS_NEEDED;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
 
-    button = XtVaCreateManagedWidget("XmDISPLAY_STATIC", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmDISPLAY_STATIC", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
@@ -485,31 +922,29 @@ Widget mw;
     String resource = XtNewString(XmNhorizontalScrollBarDisplayPolicy);
 
     rc = createButtonBox(parent, "Horizontal Display Policy");
-    XtVaSetValues(rc,
-		  XmNnumColumns, 1,
-		  XmNorientation, XmVERTICAL,
-		  XmNpacking, XmPACK_TIGHT,
-		  NULL);
 
-    button = XtVaCreateManagedWidget("XmDISPLAY_NONE", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmDISPLAY_NONE", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmDISPLAY_NONE;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
     
-    button = XtVaCreateManagedWidget("XmDISPLAY_AS_NEEDED", xmToggleButtonWidgetClass, rc,
-				     XmNset, True,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmDISPLAY_AS_NEEDED", xmToggleButtonWidgetClass, rc,
+	XmNset, True,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
     svs->value = XmDISPLAY_AS_NEEDED;
     XtAddCallback(button, XmNvalueChangedCallback, cbRadio, (XtPointer) svs);
 
-    button = XtVaCreateManagedWidget("XmDISPLAY_STATIC", xmToggleButtonWidgetClass, rc,
-				     NULL);
+    button = XtVaCreateManagedWidget(
+	"XmDISPLAY_STATIC", xmToggleButtonWidgetClass, rc,
+	NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
@@ -686,7 +1121,7 @@ Widget mw;
     String resource = XtNewString(XmNspace);
 
     scale = createScaleBox(parent, "Space", 0, 10);
-    XtVaSetValues(scale, XmNvalue, 6, NULL);
+    XtVaSetValues(scale, XmNvalue, 4, NULL);
     svs = XtNew(SetValueStruct);
     svs->mw = mw;
     svs->resource = resource;
@@ -700,40 +1135,73 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-    CellType cell;
     int x, y;
     char buf[200];
     Widget text = (Widget) client;
-    XbaeMatrixWidget mw;
     XbaeMatrixSelectCellCallbackStruct *cbs =
 	(XbaeMatrixSelectCellCallbackStruct*) call;
+    SelectModeStruct *sms;
+    unsigned char grid;
 
-    if (XtIsSubclass(w, xbaeMatrixWidgetClass))
-	mw = ( XbaeMatrixWidget )w;
-    else if (XtIsSubclass(XtParent(w), xbaeMatrixWidgetClass))
-	mw = ( XbaeMatrixWidget )XtParent(w);
-    else
-	return;
-
-    xbaeEventToXY(mw, cbs->event, &x, &y, &cell);
-
+    XtVaGetValues(w, XmNuserData, &sms,XmNgridType, &grid, NULL);
+    XbaeMatrixEventToXY(w, cbs->event, &x, &y);
+    
     sprintf(buf, "Selected cell %d,%d : %d, %d : event %d, %d",
 	    cbs->row, cbs->column, x, y,
 	    cbs->event->xbutton.x,
 	    cbs->event->xbutton.y);
 
     XbaeMatrixDeselectAll( w );
-    XbaeMatrixSelectColumn( w, cbs->column );
-    XbaeMatrixSelectRow( w, cbs->row );
+    XbaeMatrixUnhighlightAll( w );
+    UnarmAll( w, sms );
+    
+    if( cbs->row >= 0 && cbs->column >= 0 )
+    {
+	switch (sms->value)
+	{
+	    case SelectSelect:
+		XbaeMatrixSelectColumn( w, cbs->column );
+		XbaeMatrixSelectRow( w, cbs->row );
+		break;
+
+	    case SelectHighlight:
+		XbaeMatrixHighlightColumn( w, cbs->column );
+		XbaeMatrixHighlightRow( w, cbs->row );
+		break;
+
+	    case SelectArm:
+		switch (grid)
+		{
+		    default:
+			ArmCell( w, cbs->row, cbs->column, sms );
+			break;
+
+		    case XmGRID_CELL_SHADOW:
+			ArmCell( w, cbs->row, cbs->column, sms );
+			break;
+
+		    case XmGRID_ROW_SHADOW:
+			ArmRow( w, cbs->row, sms );
+			break;
+
+		    case XmGRID_COLUMN_SHADOW:
+			ArmColumn( w, cbs->column, sms );
+			break;
+		}
+		break;
+	}
+    }
 
     XmTextSetString(text, buf);
 }
 
+
 int
 main(argc, argv)
 int argc;
 char *argv[];
 {
+    SelectModeStruct *sms;
     Widget toplevel, form, rc, mw, text;
     XtAppContext app;
 #ifdef USE_EDITRES
@@ -753,91 +1221,120 @@ char *argv[];
     form = XtVaCreateWidget("form", xmFormWidgetClass, toplevel,
 			    NULL);
 
-    text = XtVaCreateManagedWidget("text", xmTextFieldWidgetClass, form,
-				   XmNeditable,		False,
-				   XmNcursorPositionVisible, False,
-				   XmNleftAttachment,	XmATTACH_FORM,
-				   XmNleftOffset,	4,
-				   XmNbottomAttachment,	XmATTACH_FORM,
-				   XmNbottomOffset,	4,
-				   XmNrightAttachment,	XmATTACH_FORM,
-				   XmNrightOffset,	4,
-				   NULL);
-    
-    mw = XtVaCreateManagedWidget("mw", xbaeMatrixWidgetClass, form,
-				 XmNtopOffset,		4,
-				 XmNleftAttachment,	XmATTACH_FORM,
-				 XmNleftOffset,		4,
-				 XmNbottomWidget,	text,
-				 XmNbottomAttachment,	XmATTACH_WIDGET,
-				 XmNbottomOffset,	4,
-				 XmNrightAttachment,	XmATTACH_FORM,
-				 XmNrightOffset,	4,
-				 NULL);
+    text = XtVaCreateManagedWidget(
+	"text", xmTextFieldWidgetClass, form,
+	XmNeditable,		False,
+	XmNcursorPositionVisible, False,
+	XmNleftAttachment,	XmATTACH_FORM,
+	XmNleftOffset,	4,
+	XmNbottomAttachment,	XmATTACH_FORM,
+	XmNbottomOffset,	4,
+	XmNrightAttachment,	XmATTACH_FORM,
+	XmNrightOffset,	4,
+	NULL);
+
+    sms = XtNew(SelectModeStruct);
+    sms->value = SelectSelect;
+
+    {
+	Widget button = XtVaCreateWidget("temp", xmToggleButtonWidgetClass,
+					 form, NULL);
+	XtVaGetValues(button,
+		      XmNselectColor, &(sms->armColor),
+		      XmNforeground, &(sms->foreColor),
+		      XmNbackground, &(sms->backColor),
+		      NULL);
+	XtDestroyWidget(button);
+    }
+	
+    mw = XtVaCreateManagedWidget(
+	"mw", xbaeMatrixWidgetClass, form,
+	XmNuserData,		(XtPointer) sms,
+	XmNtopOffset,		4,
+	XmNleftAttachment,	XmATTACH_FORM,
+	XmNleftOffset,		4,
+	XmNbottomWidget,	text,
+	XmNbottomAttachment,	XmATTACH_WIDGET,
+	XmNbottomOffset,	4,
+	XmNrightAttachment,	XmATTACH_OPPOSITE_FORM,
+	XmNrightOffset,	4,
+	NULL);
     XtOverrideTranslations(mw,
 			   XtParseTranslationTable(
 			":<Btn1Down>: SelectCell(cell) EditCell(Pointer)"));
 
     XtAddCallback(mw, XmNselectCellCallback, cbSelect, (XtPointer) text);
     
-    rc = XtVaCreateManagedWidget("rc", xmRowColumnWidgetClass, form,
-				 XmNtopAttachment,	XmATTACH_FORM,
-				 XmNleftAttachment,	XmATTACH_FORM,
-				 XmNadjustLast,		True,
-				 XmNorientation,	XmHORIZONTAL,
-				 XmNnumColumns,		2,
-				 XmNpacking,		XmPACK_TIGHT,
-				 NULL);
+    rc = XtVaCreateManagedWidget(
+	"rc", xmRowColumnWidgetClass, form,
+	XmNtopAttachment,	XmATTACH_FORM,
+	XmNleftAttachment,	XmATTACH_FORM,
+	XmNadjustLast,		True,
+	XmNorientation,		XmHORIZONTAL,
+	XmNnumColumns,		2,
+	XmNpacking,		XmPACK_TIGHT,
+	NULL);
 
     createGridType(rc, mw);
-    createPlacement(rc, mw);
-    
-    rc = XtVaCreateManagedWidget("rc", xmRowColumnWidgetClass, rc,
-				 XmNorientation,	XmVERTICAL,
-				 XmNnumColumns,		2,
-				 XmNpacking,		XmPACK_TIGHT,
-				 NULL);
-
-    createFill(rc, mw);
-    createColumnResize(rc, mw);
-
-    rc = XtVaCreateManagedWidget("rc", xmRowColumnWidgetClass, form,
-				 XmNtopWidget,		rc,
-				 XmNtopAttachment,	XmATTACH_WIDGET,
-				 XmNleftAttachment,	XmATTACH_FORM,
-				 XmNrightAttachment,	XmATTACH_FORM,
-				 XmNadjustLast,		True,
-				 XmNorientation,	XmHORIZONTAL,
-				 XmNnumColumns,		1,
-				 XmNpacking,		XmPACK_COLUMN,
-				 NULL);
-
     createVerticalDisplay(rc, mw);
-    createHorizontalDisplay(rc, mw);
+    
+    rc = XtVaCreateManagedWidget(
+	"rc", xmRowColumnWidgetClass, form,
+	XmNtopWidget,		rc,
+	XmNtopAttachment,	XmATTACH_WIDGET,
+	XmNleftAttachment,	XmATTACH_FORM,
+	XmNrightAttachment,	XmATTACH_FORM,
+	XmNadjustLast,		True,
+	XmNorientation,		XmHORIZONTAL,
+	XmNnumColumns,		4,
+	XmNpacking,		XmPACK_TIGHT,
+	NULL);
+
+    createPlacement(rc, mw);
     createCellShadowType(rc, mw);
     createShadowType(rc, mw);
+    createHorizontalDisplay(rc, mw);
 
-    rc = XtVaCreateManagedWidget("rc", xmRowColumnWidgetClass, form,
-				 XmNtopWidget,		rc,
-				 XmNtopAttachment,	XmATTACH_WIDGET,
-				 XmNleftAttachment,	XmATTACH_FORM,
-				 XmNrightAttachment,	XmATTACH_FORM,
-				 XmNadjustLast,		True,
-				 XmNorientation,	XmHORIZONTAL,
-				 XmNnumColumns,		4,
-				 XmNpacking,		XmPACK_COLUMN,
-				 NULL);
+    rc = XtVaCreateManagedWidget(
+	"rc", xmRowColumnWidgetClass, form,
+	XmNtopWidget,		rc,
+	XmNtopAttachment,	XmATTACH_WIDGET,
+	XmNleftAttachment,	XmATTACH_FORM,
+	XmNrightAttachment,	XmATTACH_FORM,
+	XmNadjustLast,		False,
+	XmNorientation,		XmVERTICAL,
+	XmNnumColumns,		3,
+	XmNpacking,		XmPACK_COLUMN,
+	NULL);
     
     createCellShadowThickness(rc, mw);
     createShadowThickness(rc, mw);
-    createSpace(rc, mw);
     createFixedRows(rc, mw);
     createFixedColumns(rc, mw);
     createCellMarginWidth(rc, mw);
+    createCellMarginHeight(rc, mw);
     createTrailingFixedRows(rc, mw);
     createTrailingFixedColumns(rc, mw);
-    createCellMarginHeight(rc, mw);
+    createFill(rc, mw);
+    createColumnResize(rc, mw);
+    createButtonLabels(rc, mw);
+    createRowAttached(rc, mw);
+
+    rc = XtVaCreateManagedWidget(
+	"rc", xmRowColumnWidgetClass, form,
+	XmNtopWidget,		rc,
+	XmNtopAttachment,	XmATTACH_WIDGET,
+	XmNleftAttachment,	XmATTACH_FORM,
+	XmNrightAttachment,	XmATTACH_FORM,
+	XmNadjustLast,		False,
+	XmNorientation,		XmHORIZONTAL,
+	XmNnumColumns,		4,
+	XmNpacking,		XmPACK_TIGHT,
+	NULL);
     
+    createSpace(rc, mw);
+    createSelectMode(rc, mw);
+
     LoadMatrix(mw);
 
     XtVaSetValues(mw,
