@@ -18,8 +18,14 @@
  * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
+ *
+ * $Id: colors.c,v 1.8 1999/07/24 01:49:10 lister Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <X11/StringDefs.h>
 #include <Xm/Form.h>
@@ -152,7 +158,6 @@ char *argv[];
 				NULL);
     XtAddCallback(colorData.matrix, XmNleaveCellCallback, SetColorCB,
 		  (XtPointer)&colorData);
-
     /*
      * Create a button to reset XmNcolors
      */
@@ -248,7 +253,7 @@ XbaeMatrixLeaveCellCallbackStruct *call_data;
     fromVal.size = strlen(call_data->value) + 1;
     fromVal.addr = call_data->value;
     toVal.size = sizeof(Pixel);
-    toVal.addr = (caddr_t) &color;
+    toVal.addr = (XtPointer) &color;
     if (!XtConvertAndStore(w, XtRString, &fromVal, XtRPixel, &toVal))
 	return;
 
