@@ -18,8 +18,14 @@
  * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
+ *
+ * $Id: dynamic.c,v 1.5 1999/07/24 01:49:10 lister Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <Xm/RowColumn.h>
 #include <Xm/Label.h>
 #include <Xm/PushB.h>
