@@ -18,14 +18,25 @@
  * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
+ *
+ * $Id: select-drag.c,v 1.8 1999/07/24 01:49:10 lister Exp $
  */
 
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <Xm/Xm.h>
+#include <Xm/Form.h>
+#include <Xm/PushB.h>
 #include <Xbae/Matrix.h>
+#include <Xm/PanedW.h>
 
 void LoadMatrix();
 void SelectCB();
+void ResetCB();
 
 /*
  * Create an uneditable matrix. When Btn1 is dragged, a rectangular region
@@ -37,26 +48,39 @@ main(argc, argv)
 int argc;
 char *argv[];
 {
-    Widget toplevel, matrix;
+    Widget toplevel, matrix, pb, pane;
     XtAppContext app;
 #ifdef USE_EDITRES
     extern void _XEditResCheckMessages();
 #endif
 
-    toplevel = XtAppInitialize(&app, "Select-drag",
-			       NULL, 0,
-			       &argc, argv,
-			       NULL,
-			       NULL, 0);
+    toplevel = XtAppInitialize(
+	&app, "Select-drag", NULL, 0, &argc, argv, NULL, NULL, 0);
 #ifdef USE_EDITRES
-    XtAddEventHandler( toplevel, (EventMask)0, True,
+    XtAddEventHandler(toplevel, (EventMask)0, True,
                        _XEditResCheckMessages, NULL);
 #endif
 
-    matrix = XtVaCreateManagedWidget("matrix",
-				     xbaeMatrixWidgetClass, toplevel,
-				     NULL);
+    pane = XtVaCreateManagedWidget(
+	"pane", xmPanedWindowWidgetClass, toplevel,
+	NULL);
+
+    pb = XtVaCreateManagedWidget(
+	"Reset", xmPushButtonWidgetClass, pane,
+	NULL);
+
+    matrix = XtVaCreateManagedWidget(
+	"matrix", xbaeMatrixWidgetClass, pane,
+	XmNtopAttachment, XmATTACH_FORM,
+	XmNbottomAttachment, XmATTACH_FORM,
+	XmNleftAttachment, XmATTACH_FORM,
+	XmNtopAttachment, XmATTACH_FORM,
+	NULL);
+
     XtAddCallback(matrix, XmNselectCellCallback, SelectCB, NULL);
+
+    XtAddCallback(pb, XmNactivateCallback, ResetCB, (XtPointer)matrix);
+    
     LoadMatrix(matrix);
 
     XtRealizeWidget(toplevel);
@@ -74,22 +98,26 @@ Widget w;
 XtPointer client_data;
 XbaeMatrixSelectCellCallbackStruct *call_data;
 {
-    static int start_row, start_column;
-    static int last_row, last_column;
+    static int start_row = -1, start_column = -1;
+    static int last_row = -1, last_column = -1;
     int i, j;
 
     /*
-     * If we have params, we must be starting a rectangle
+     * Avoid scrolling all over the place when selecting cells
      */
-    if (call_data->num_params) {
+    /*XtVaSetValues(w, XmNselectScrollVisible, False, NULL);*/
+
+    if (call_data->num_params == 0)
+    {
 	start_row = last_row = call_data->row;
 	start_column = last_column = call_data->column;
 	XbaeMatrixSelectCell(w, last_row, last_column);
     }
     /*
-     * Else we are in the middle of dragging a rectangle
+     * Else we might be in an extended drag operation
      */
-    else {
+    else if (!strcmp(call_data->params[ 0 ], "extend"))
+    {
 	/*
 	 * This is a pretty simplistic, stupid implementation
 	 * of selecting a rectangle.
@@ -104,10 +132,11 @@ XbaeMatrixSelectCellCallbackStruct *call_data;
 		for (j = last_column + 1; j <= call_data->column; j++)
 		    XbaeMatrixSelectCell(w, i, j);
 	}
-	
 	last_row = call_data->row;
 	last_column = call_data->column;
+printf("%d, %d\n", last_row, last_column);
     }
+    /*XtVaSetValues(w, XmNselectScrollVisible, True, NULL);*/
 }
 
 /* ARGSUSED */
@@ -145,3 +174,14 @@ Widget matrix;
     }
     XtFree((XtPointer)cells);
 }
+
+void
+ResetCB(w, cd, cb)
+Widget w;
+XtPointer cd;
+XtPointer cb;
+{
+    Widget mw = (Widget)cd;
+
+    XbaeMatrixDeselectAll(mw);
+}
