@@ -3,16 +3,22 @@
  *
  * WARNING: This example uses some private xbae routines.
  *
- * $Id: select-push.c,v 1.3 1997/02/08 06:04:58 lister Exp $
+ * $Id: select-push.c,v 1.4 1999/07/24 01:49:10 lister Exp $
  */
 
 /*
+ * WARNING
  * This program demonstrates a possible use of the individual
  * cell and row shadow types resources. However, it does this
  * in a rather improper fashion, by using a few of the matrix's
  * internal functions. But the overall effect is rather slick :).
  */
 
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <Xbae/Matrix.h>
 #include <Xbae/MatrixP.h>
 #include <Xbae/Utils.h>
