@@ -1,5 +1,7 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
+ * Copyright(c) 1999 Andrew Lister
+ *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
  * any purpose and without fee is hereby granted, provided that the
@@ -20,14 +22,19 @@
  * ING TO THE SOFTWARE.
  *
  * CaptionWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
+ * 
+ * $Id: Caption.c,v 1.12 1999/07/24 02:03:48 lister Exp $
  */
 
 /*
  * Caption.c - displays a caption label next to it's child.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <ctype.h>
-#include <X11/Xos.h>
 #include <X11/StringDefs.h>
 #include <Xm/XmP.h>
 #include <Xm/Label.h>
@@ -78,52 +85,40 @@ static XtResource resources[] = {
 /*
  * Declaration of methods
  */
-static void ClassInitialize P(( void ));
-static void Initialize P(( XbaeCaptionWidget request, XbaeCaptionWidget new,
-			   ArgList args, Cardinal *num_args ));
-static void InsertChild P(( Widget w ));
-static Boolean SetValues P(( XbaeCaptionWidget current,
-			     XbaeCaptionWidget request, XbaeCaptionWidget new,
-			     ArgList args, Cardinal *num_args ));
-static void SetValuesAlmost P(( XbaeCaptionWidget old, XbaeCaptionWidget new,
-				XtWidgetGeometry *request,
-				XtWidgetGeometry *reply ));
-static void GetValuesHook P(( XbaeCaptionWidget cw, ArgList args,
-			      Cardinal *num_args ));
-static void Resize P(( XbaeCaptionWidget cw ));
-static void ChangeManaged P(( XbaeCaptionWidget cw ));
-static XtGeometryResult GeometryManager P(( Widget w,
-					    XtWidgetGeometry *desired,
-					    XtWidgetGeometry *allowed ));
-static XtGeometryResult QueryGeometry P(( XbaeCaptionWidget cw,
-					  XtWidgetGeometry *proposed,
-					  XtWidgetGeometry *desired ));
+static void ClassInitialize P((void));
+static void Initialize P((Widget, Widget, ArgList, Cardinal *));
+static void InsertChild P((Widget));
+static Boolean SetValues P((Widget, Widget, Widget, ArgList, Cardinal *));
+static void SetValuesAlmost P((Widget, Widget, XtWidgetGeometry *,
+			       XtWidgetGeometry *));
+static void GetValuesHook P((Widget, ArgList, Cardinal *));
+static void Resize P((Widget));
+static void ChangeManaged P((Widget));
+static XtGeometryResult GeometryManager P((Widget, XtWidgetGeometry *,
+					   XtWidgetGeometry *));
+static XtGeometryResult QueryGeometry P((Widget, XtWidgetGeometry *,
+					 XtWidgetGeometry *));
 
 /*
  * Private functions
  */
-static void ComputeSize P(( XbaeCaptionWidget cw, Dimension *cwWidth,
-			    Dimension *cwHeight, Dimension childWidth,
-			    Dimension cHeight, Dimension cBorderWidth ));
-static void ComputeUserChildSize P(( XbaeCaptionWidget cw, Dimension cwWidth,
-				     Dimension cwHeight, Dimension *childWidth,
-				     Dimension *childHeight,
-				     Dimension childBorderWidth ));
+static void ComputeSize P((XbaeCaptionWidget, Dimension *, Dimension *,
+			    Dimension, Dimension, Dimension));
+static void ComputeUserChildSize P((XbaeCaptionWidget, Dimension, Dimension,
+				     Dimension *, Dimension *, Dimension));
 
-static void Layout P(( XbaeCaptionWidget cw, Boolean configure ));
-static Boolean CompareStrings P(( String in, String test ));
+static void Layout P((XbaeCaptionWidget, Boolean));
+static Boolean CompareStrings P((String, String));
 
 /*
  * Type converters
  */
-static Boolean CvtStringToLabelPosition P(( Display *dpy, XrmValuePtr args,
-					    Cardinal *num_args,
-					    XrmValuePtr from, XrmValuePtr to,
-					    XtPointer *data ));
-static Boolean CvtStringToLabelAlignment P(( Display *dpy, XrmValuePtr args,
-					     Cardinal *num_args,
-					     XrmValuePtr from, XrmValuePtr to,
-					     XtPointer *data ));
+static Boolean CvtStringToLabelPosition P((Display *, XrmValuePtr, Cardinal *,
+					    XrmValuePtr, XrmValuePtr,
+					    XtPointer *));
+static Boolean CvtStringToLabelAlignment P((Display *, XrmValuePtr,
+					     Cardinal *, XrmValuePtr,
+					     XrmValuePtr, XtPointer *));
 
 
 XbaeCaptionClassRec xbaeCaptionClassRec = {
@@ -135,7 +130,7 @@ XbaeCaptionClassRec xbaeCaptionClassRec = {
 	/* class_initialize		*/ ClassInitialize,
 	/* class_part_initialize	*/ NULL,
 	/* class_inited			*/ False,
-	/* initialize			*/ ( XtInitProc )Initialize,
+	/* initialize			*/ (XtInitProc)Initialize,
 	/* initialize_hook		*/ NULL,
 	/* realize			*/ XtInheritRealize,
 	/* actions			*/ NULL,
@@ -148,24 +143,24 @@ XbaeCaptionClassRec xbaeCaptionClassRec = {
 	/* compress_enterleave		*/ True,
 	/* visible_interest		*/ False,
 	/* destroy			*/ NULL,
-	/* resize			*/ ( XtWidgetProc )Resize,
+	/* resize			*/ (XtWidgetProc)Resize,
 	/* expose			*/ _XmRedisplayGadgets,
-	/* set_values			*/ ( XtSetValuesFunc )SetValues,
+	/* set_values			*/ (XtSetValuesFunc)SetValues,
 	/* set_values_hook		*/ NULL,
-	/* set_values_almost		*/ ( XtAlmostProc )SetValuesAlmost,
-	/* get_values_hook		*/ ( XtArgsProc )GetValuesHook,
+	/* set_values_almost		*/ (XtAlmostProc)SetValuesAlmost,
+	/* get_values_hook		*/ (XtArgsProc)GetValuesHook,
 	/* accept_focus			*/ NULL,
 	/* version			*/ XtVersion,
 	/* callback_private		*/ NULL,
 	/* tm_table			*/ XtInheritTranslations,
-	/* query_geometry		*/ ( XtGeometryHandler )QueryGeometry,
+	/* query_geometry		*/ (XtGeometryHandler)QueryGeometry,
 	/* display_accelerator		*/ NULL,
 	/* extension			*/ NULL
     },
     {
     /* composite_class fields */
 	/* geometry_manager		*/ GeometryManager,
-	/* change_managed		*/ ( XtWidgetProc )ChangeManaged,
+	/* change_managed		*/ (XtWidgetProc)ChangeManaged,
 	/* insert_child			*/ InsertChild,
 	/* delete_child			*/ XtInheritDeleteChild,
 	/* extension			*/ NULL,
@@ -213,11 +208,12 @@ ClassInitialize()
 
 /* ARGSUSED */
 static void
-Initialize(request, new, args, num_args)
-XbaeCaptionWidget request, new;
+Initialize(req, n, args, num_args)
+Widget req, n;
 ArgList args;
 Cardinal *num_args;
 {
+    XbaeCaptionWidget new = (XbaeCaptionWidget)n;
     /*
      * Validate our labelPosition
      */
@@ -310,11 +306,13 @@ Widget w;
 
 /* ARGSUSED */
 static Boolean
-SetValues(current, request, new, args, num_args)
-XbaeCaptionWidget current, request, new;
+SetValues(cur, req, nw, args, num_args)
+Widget cur, req, nw;
 ArgList args;
 Cardinal *num_args;
 {
+    XbaeCaptionWidget current = (XbaeCaptionWidget)cur;
+    XbaeCaptionWidget new = (XbaeCaptionWidget)nw;
     Dimension old_label_width, old_label_height;
     Boolean layout = False;
     int n;
@@ -452,12 +450,13 @@ Cardinal *num_args;
 
 /* ARGSUSED */
 static void
-SetValuesAlmost(old, new, request, reply)
-XbaeCaptionWidget old;
-XbaeCaptionWidget new;
+SetValuesAlmost(o, n, request, reply)
+Widget o;
+Widget n;
 XtWidgetGeometry *request;
 XtWidgetGeometry *reply;
 {
+    XbaeCaptionWidget new = (XbaeCaptionWidget)n;
     /*
      * If XtGeometryAlmost, accept compromize - our resize method
      * will take care of it.
@@ -479,11 +478,12 @@ XtWidgetGeometry *reply;
 }
 
 static void
-GetValuesHook(cw, args, num_args)
-XbaeCaptionWidget cw;
+GetValuesHook(w, args, num_args)
+Widget w;
 ArgList args;
 Cardinal *num_args;
 {
+    XbaeCaptionWidget cw = (XbaeCaptionWidget)w;
     int i;
 
     /*
@@ -638,10 +638,10 @@ Boolean configure;
 }
 
 static void
-Resize(cw)
-XbaeCaptionWidget cw;
+Resize(w)
+Widget w;
 {
-    Layout(cw, True);
+    Layout((XbaeCaptionWidget)w, True);
 }
 
 /*
@@ -652,7 +652,7 @@ static void
 #if NeedFunctionPrototypes
 ComputeUserChildSize(XbaeCaptionWidget cw, Dimension cwWidth,
 		     Dimension cwHeight, Dimension *childWidth,
-		     Dimension *childHeight, Dimension childBorderWidth )
+		     Dimension *childHeight, Dimension childBorderWidth)
 #else
 ComputeUserChildSize(cw, cwWidth, cwHeight, childWidth, childHeight,
 		     childBorderWidth)
@@ -760,9 +760,10 @@ Dimension childBorderWidth;
 }
 
 static void
-ChangeManaged(cw)
-XbaeCaptionWidget cw;
+ChangeManaged(w)
+Widget w;
 {
+    XbaeCaptionWidget cw = (XbaeCaptionWidget)w;
     Dimension width, height;
     XtGeometryResult result;
 
@@ -946,10 +947,11 @@ XtWidgetGeometry *desired, *allowed;
 }
 
 static XtGeometryResult
-QueryGeometry(cw, proposed, desired)
-XbaeCaptionWidget cw;
+QueryGeometry(w, proposed, desired)
+Widget w;
 XtWidgetGeometry *proposed, *desired;
 {
+    XbaeCaptionWidget cw = (XbaeCaptionWidget)w;
 #define Set(bit) (proposed->request_mode & bit)
 
     /*
@@ -1214,3 +1216,13 @@ XtPointer *data;
 
     return True;
 }
+
+Widget
+XbaeCreateCaption(parent, name, args, ac) 
+Widget parent;
+String name;
+ArgList args;
+Cardinal ac;
+{ 
+    return XtCreateWidget(name, xbaeCaptionWidgetClass, parent, args, ac); 
+} 
