@@ -1,5 +1,6 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
+ * Copyright(c) 1999 Andrew Lister
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
  * any purpose and without fee is hereby granted, provided that the
@@ -20,15 +21,13 @@
  * ING TO THE SOFTWARE.
  *
  * CaptionWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
+ * 
+ * $Id: Caption.h,v 1.8 1999/07/24 02:03:48 lister Exp $
  */
 
 #ifndef _Xbae_Caption_h
 #define _Xbae_Caption_h
 
-/*
- *  "@(#)Caption.h	4.1 3/19/95"
- */
-
 /*
  * Caption Widget public include file
  */
@@ -38,6 +37,24 @@
 #include <Xbae/patchlevel.h>
 
 
+#ifndef XlibSpecificationRelease
+# ifndef _XFUNCPROTOBEGIN
+#   ifdef __cplusplus                      /* for C++ V2.0 */
+#     define _XFUNCPROTOBEGIN extern "C" {   /* do not leave open across includes */
+#     define _XFUNCPROTOEND }
+#   else
+#     define _XFUNCPROTOBEGIN
+#     define _XFUNCPROTOEND
+#   endif
+# endif /* _XFUNCPROTOBEGIN */
+#else
+#include <X11/Xfuncproto.h>
+#endif
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* Resources:
  * Name			Class		RepType		Default Value
  * ----			-----		-------		-------------
@@ -55,16 +72,34 @@
 /*
  * New resource constants
  */
+#ifndef XmNlabelPosition
 #define XmNlabelPosition "labelPosition"
+#endif
+#ifndef XmCLabelPosition
 #define XmCLabelPosition "LabelPosition"
+#endif
+#ifndef XmNlabelAlignment
 #define XmNlabelAlignment "labelAlignment"
+#endif
+#ifndef XmCLabelAlignment
 #define XmCLabelAlignment "LabelAlignment"
+#endif
+#ifndef XmNlabelTextAlignment
 #define XmNlabelTextAlignment "labelTextAlignment"
+#endif
+#ifndef XmNlabelOffset
 #define XmNlabelOffset "labelOffset"
+#endif
+#ifndef XmCLabelOffset
 #define XmCLabelOffset "LabelOffset"
+#endif
 
+#ifndef XmRLabelPosition
 #define XmRLabelPosition "LabelPosition"
+#endif
+#ifndef XmRLabelAlignment
 #define XmRLabelAlignment "LabelAlignment"
+#endif
 
 /* Class record constants */
 
@@ -73,6 +108,26 @@ extern WidgetClass xbaeCaptionWidgetClass;
 typedef struct _XbaeCaptionClassRec *XbaeCaptionWidgetClass;
 typedef struct _XbaeCaptionRec *XbaeCaptionWidget;
 
+/*
+ * Prototype wrapper
+ */
+#ifndef P
+#if defined(__STDC__) || defined (__cplusplus)
+#define P(x)		x
+#else
+#define P(x)		()
+#define const
+#define volatile
+#endif
+#endif
+
+/*
+ * External interfaces to class methods
+ */
+_XFUNCPROTOBEGIN
+extern Widget XbaeCreateCaption P((Widget, String, ArgList, Cardinal));
+_XFUNCPROTOEND
+
 /*
  * Type for XmNlabelPosition resource
  */
@@ -92,6 +147,20 @@ typedef enum _XbaeLabelAlignment {
     XbaeAlignmentBottomOrRight
 } XbaeLabelAlignment;
 
+/* provide clean-up for those with R4 */
+#ifndef XlibSpecificationRelease
+# undef _Xconst
+# undef _XFUNCPROTOBEGIN
+# undef _XFUNCPROTOEND
+#endif
+
+#ifndef XbaeIsXbaeCaption
+#define XbaeIsXbaeCaption(w)    XtIsSubclass(w, xbaeCaptionWidgetClass)
+#endif /* XbaeIsXbaeInput */
+
+#ifdef __cplusplus
+}  /* Close scope of 'extern "C"' declaration which encloses file. */
+#endif
+
 #endif /* _Xbae_Caption_h */
-/* DON'T ADD STUFF AFTER THIS #endif */
 
