@@ -1,5 +1,6 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
+ * Copyright(c) 1995-99 Andrew Lister
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
  * any purpose and without fee is hereby granted, provided that the
@@ -20,22 +21,27 @@
  * ING TO THE SOFTWARE.
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
+ *
+ * $Id: Converters.c,v 1.12 1999/07/24 02:03:48 lister Exp $
  */
 
 /*
  * Convert a comma separated list of strings to a NULL terminated array
- * of substrings.  Handles escaped commas (\,) and escaped escaped commas (\\,)
- * - A comma (,) terminates a string
- * - A backslash-comma (\,) does not terminate a string and is copied as
- *   a comma (,)
- * - A backslash-backslash-comma (\\,) does not terminate a string and is
- *   copied as a backslash-comma (\,)
+ * of substrings.
+ * A comma (,) terminates a string
+ * Backslash is an escape character allowing leading and terminating
+ * white space to be protected.  A backslash-comma (\,) does not
+ * terminate a string and is copied as a comma (,).
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <X11/Xos.h>
+/* #include <X11/Xos.h> */
 #include <X11/StringDefs.h>
 #include <Xm/XmP.h>
 #include <Xm/AtomMgr.h>
@@ -46,7 +52,7 @@
 #define tolower(c)      ((c) - 'A' + 'a')
 #endif
 
-static Boolean StringsAreEqual P(( String in, String test, int length ));
+static Boolean StringsAreEqual P((String, String, int));
 
 /* ARGSUSED */
 Boolean
@@ -60,7 +66,7 @@ XtPointer *data;
     static String *array;
     String start = from->addr;
 
-    if (*num_args != 0)
+     if (*num_args != 0)
 	XtAppWarningMsg(
 	    XtDisplayToApplicationContext(dpy),
 	    "cvtStringToStringArray", "wrongParameters",
@@ -95,17 +101,7 @@ XtPointer *data;
 	     */
 	    if (*ch == '\\')
 	    {
-		/*
-		 * Take care of backslash-backslash-comma
-		 */
-		if (*(ch + 1) == '\\' && *(ch + 2) == ',')
-		    ch += 2;
-
-		/*
-		 * Take care of backslash-comma
-		 */
-		else if (*(ch + 1) == ',')
-		    ch++;
+		ch++;
 	    }
 	    /*
 	     * We hit an unescaped comma
@@ -141,27 +137,16 @@ XtPointer *data;
 		 */
 		if (*ch == '\\')
 		{
-		    /*
-		     * Take care of backslash-backslash-comma
-		     */
-		    if (*(ch + 1) == '\\' && *(ch + 2) == ',')
+		    ch++;
+
+		    if (*ch == '\0')
 		    {
-			len += 2;
-			ch += 2;
+			break;
 		    }
-		    /*
-		     * Take care of backslash-comma
-		     */
-		    else if (*(ch + 1) == ',')
-		    {
-			len++;
-			ch++;
-		    }
-		    else
-			len++;
+
 		}
-		else
-		    len++;
+
+		len++;
 	    }
 
 	    /*
@@ -170,10 +155,10 @@ XtPointer *data;
 	    next = ch + 1;
 
 	    /*
-	     * Back up over trailing white space if we moved at all
+	     * Back up over unprotected trailing white space if we moved at all
 	     */
 	    if (ch != start)
-		while (isspace((unsigned char) *(--ch)))
+		while (*(ch-2) != '\\' && isspace((unsigned char) *(--ch)))
 		    len--;
 
 	    /*
@@ -183,8 +168,6 @@ XtPointer *data;
 
 	    /*
 	     * Copy the substring into our new string.
-	     * backslash-comma gets copied as comma.
-	     * backslash-backslash-comma gets copied as backslash-comma.
 	     */
 	    for (i = 0, ch = start, a = array[sub];
 		 i < len;
@@ -196,29 +179,10 @@ XtPointer *data;
 		 */
 		if (*ch == '\\')
 		{
-		    /*
-		     * Take care of backslash-backslash-comma
-		     */
-		    if (*(ch + 1) == '\\' && *(ch + 2) == ',')
-		    {
-			*(a++) = '\\';
-			*(a++) = ',';
-			i++;
-			ch += 2;
-		    }
-		    /*
-		     * Take care of backslash-comma
-		     */
-		    else if (*(ch + 1) == ',')
-		    {
-			*(a++) = ',';
-			ch++;
-		    }
-		    else
-			*(a++) = *ch;
+		    ch++;
 		}
-		else
-		    *(a++) = *ch;
+
+		*(a++) = *ch;
 	    }
 	    *a = '\0';
 
@@ -301,23 +265,22 @@ XtPointer *data;
 	 */
 	for (ch = start, count = 1; *ch != '\0'; ch++)
 	{
-	    if (*ch == '\\' && *(ch+1) == 'n' )
+	    if ((*ch == '\\' && *(ch+1) == 'n') || *ch == '\n')
 	        count++;
 
-
         }
 
 	/*
 	 * Malloc the array
 	 */
 	array = (String **)XtMalloc((count + 1) * sizeof(String *));
-	array[count] = (String* )0;
+	array[count] = (String*)0;
 	
-	for (k = 0; k < count; k++ )
+	for (k = 0; k < count; k++)
 	{
 	    for (ch = start; *ch != '\0' ; ch++)
 	    {
-	        if (*ch == '\\' && *(ch+1) == 'n' )
+	        if ((*ch == '\\' && *(ch+1) == 'n') || *ch == '\n')
 	            break;
 	    }
 	    c = *ch ; *ch = '\0';
@@ -327,7 +290,7 @@ XtPointer *data;
             lto.addr = (char *)(&array[k]);
             lto.size = sizeof(String *);
             
-            if ( !CvtStringToStringArray(dpy, args, num_args, &lfrom, &lto, data))
+            if (!CvtStringToStringArray(dpy, args, num_args, &lfrom, &lto, data))
             {
                 *ch = c;
 		XtDisplayStringConversionWarning(dpy, from->addr,
@@ -338,18 +301,17 @@ XtPointer *data;
        		return False;
 	    }
             *ch = c;
-	    if ( c == '\\' )
-	    {
-	        ch++ ;
-	        start = ch+1;
-	    }
+	    if (c == '\\')
+	        start = ch+2 ;
+	    else if (c == '\n')
+	    	start = ch+1;
 	    else
-	        start = ch;
+	    	start = ch;
         }
     }
 
     if (to->addr == NULL)
-	to->addr = (caddr_t) &array;
+	to->addr = (XtPointer) &array;
     else
 	*(String ***) to->addr = array;
     to->size = sizeof(String **);
@@ -615,6 +577,123 @@ int length;
     return True;
 }
 
+
+/*
+ * Convert a comma separated list of alignments to array of Booleans.
+ */
+/* ARGSUSED */
+Boolean
+CvtStringToBooleanArray(dpy, args, num_args, from, to, data)
+Display *dpy;
+XrmValuePtr args;
+Cardinal *num_args;
+XrmValuePtr from, to;
+XtPointer *data;
+{
+    static Boolean *array;
+    String start = from->addr;
+    char *ch;
+    int i, count;
+
+    if (*num_args != 0)
+	XtAppWarningMsg(
+	    XtDisplayToApplicationContext(dpy),
+	    "cvtStringToBooleanArray", "wrongParameters",
+	    "XbaeMatrix",
+	    "String to BooleanArray conversion needs no extra arguments",
+	    NULL, NULL);
+
+    /*
+     * User didn't provide enough space
+     */
+    if (to->addr != NULL && to->size < sizeof(Boolean *))
+    {
+	to->size = sizeof(Boolean *);
+
+	return False;
+    }
+
+    if (start == NULL || *start == '\0')
+	array = NULL;
+
+    else
+    {
+
+	/*
+	 * Count the comma separated alignments
+	 */
+	for (ch = start, count = 1; *ch != '\0'; ch++)
+	    if (*ch == ',')
+		count++;
+
+	/*
+	 * Malloc the array
+	 */
+	array = (Boolean*) XtMalloc((count + 1) * sizeof(Boolean));
+
+/*	array[count] = BAD_WIDTH;
+*/
+	for (i = 0; i < count; i++)
+	{
+  /*
+	     * Skip leading white space
+	     */
+	    while (isspace(*start))
+		start++;
+
+	    if (StringsAreEqual(start, "true", 4))
+		array[i] = True;
+	    else if (StringsAreEqual(start, "1", 1))
+		array[i] = True;
+	    else if (StringsAreEqual(start, "false", 5))
+		array[i] = False;
+	    else if (StringsAreEqual(start, "0", 1))
+		array[i] = False;
+	    else
+	    {
+		XtDisplayStringConversionWarning(dpy, from->addr,
+						 XmRBooleanArray);
+		XtFree((void*)array);
+		return False;
+	    }
+
+	    /*
+	     * Find the comma at the end of this short
+	     */
+	    /* EMPTY */
+	    for (; *start != '\0' && *start != ','; start++);
+	    start++;
+	}
+    }
+
+    if (to->addr == NULL)
+	to->addr = (XtPointer) & array;
+    else
+	*(Boolean **) to->addr = array;
+    to->size = sizeof(Boolean *);
+
+    return True;
+}
+
+
+/*
+ * Free the alignment array allocated by the String to BooleanArray converter
+ */
+/* ARGSUSED */
+void
+BooleanArrayDestructor(app, to, converter_data, args, num_args)
+XtAppContext app;
+XrmValuePtr to;
+XtPointer converter_data;
+XrmValuePtr args;
+Cardinal *num_args;
+{
+    Boolean *array = *(Boolean **) to->addr;
+
+    XtFree((XtPointer) array);
+}
+
+
 /*
  * Convert a comma separated list of alignments to array of alignments
  * (unsigned chars).  The array is terminated by BAD_ALIGNMENT.
@@ -784,10 +863,10 @@ XtPointer *data;
 	{
 	    if (*ch == ',')
 		k++;
-	    if (*ch == '\\' && *(ch+1) == 'n' )
+	    if ((*ch == '\\' && *(ch+1) == 'n') || *ch == '\n')
 	    {
 	        count_y++;
-	        if ( k > count_x )
+	        if (k > count_x)
 	            count_x = k;
 	        k = 1;
 	    }
@@ -797,9 +876,9 @@ XtPointer *data;
 	 * Malloc the arrays
 	 */
 	array = (Pixel **)XtMalloc((count_y + 1) * sizeof(Pixel *));
-	array[count_y] = (Pixel* )0;
+	array[count_y] = (Pixel*)0;
 	
-	for (k = 0; k < count_y; k++ )
+	for (k = 0; k < count_y; k++)
 	{
           row = (Pixel *)XtMalloc((count_x + 1) * sizeof(Pixel));
 	  row[count_x] = BAD_PIXEL;
@@ -811,7 +890,7 @@ XtPointer *data;
 	   */
 	  for (i = 0; i < count_x; i++) {
 
-	    if ( *start == '\0' )
+	    if (*start == '\0')
 	        break;
 	    /*
 	     * Skip leading white space
@@ -825,7 +904,7 @@ XtPointer *data;
 	    /* EMPTY */
 	    for (ch = start; *ch != '\0' && *ch != ','; ch++)
 	    {
-	        if (*ch == '\\' && *(ch+1) == 'n' )
+	        if ((*ch == '\\' && *(ch+1) == 'n') || *ch == '\n')
 	            break;
 	    }
 	    c = *ch ; *ch = '\0';
@@ -835,7 +914,7 @@ XtPointer *data;
             lto.addr = (char *)(&row[i]);
             lto.size = sizeof(Pixel);
             
-            if ( ! XtCvtStringToPixel(dpy, args, num_args, &lfrom, &lto, data))
+            if (! XtCvtStringToPixel(dpy, args, num_args, &lfrom, &lto, data))
             {
                 row[i] = last_pixel;
 		XtDisplayStringConversionWarning(dpy, from->addr,
@@ -843,12 +922,12 @@ XtPointer *data;
 	    }
 	    last_pixel = row[i];
             *ch = c;
-	    if ( c == '\0' )
+	    if (c == '\0')
 	    {
 	        start = ch;
 	        break;
 	    }
-	    if ( c == '\\' )
+	    if (c == '\\')
 	    {
 	        ch++ ;
 	        start = ch+1;
@@ -856,7 +935,7 @@ XtPointer *data;
 	    }
 	    start = ch+1;
 	  }
-	  for (  ; i < count_x-1 ; i++ )
+	  for (; i < count_x-1 ; i++)
 	  {
 	     /* fill rest of row with same value */
 	     row[i+1] = row[i];	            
@@ -865,7 +944,7 @@ XtPointer *data;
     }
 
     if (to->addr == NULL)
-	to->addr = (caddr_t) &array;
+	to->addr = (XtPointer) &array;
     else
 	*(Pixel ***) to->addr = array;
     to->size = sizeof(Pixel *);
@@ -888,10 +967,10 @@ Cardinal *num_args;
     Pixel **array = *(Pixel ***) to->addr;
     Pixel **col;
 
-    if ( array )
+    if (array)
     {
-        for ( col = array  ; col ; col++ )
-            XtFree((XtPointer)*col );
+        for (col = array  ; col ; col++)
+            XtFree((XtPointer)*col);
         XtFree((XtPointer) array);
     }
 }
@@ -931,21 +1010,39 @@ XtPointer *data;
 
     if (StringsAreEqual(start, "grid_none", 9))
 	grid_type = XmGRID_NONE;
+    else if (StringsAreEqual(start, "grid_cell_line", 14))
+	grid_type = XmGRID_CELL_LINE;
+    else if (StringsAreEqual(start, "grid_cell_shadow", 16))
+	grid_type = XmGRID_CELL_SHADOW;
+    else if (StringsAreEqual(start, "grid_row_line", 13))
+	grid_type = XmGRID_ROW_LINE;
+    else if (StringsAreEqual(start, "grid_row_shadow", 15))
+	grid_type = XmGRID_ROW_SHADOW;
+    else if (StringsAreEqual(start, "grid_column_line", 16))
+	grid_type = XmGRID_COLUMN_LINE;
+    else if (StringsAreEqual(start, "grid_column_shadow", 15))
+	grid_type = XmGRID_COLUMN_SHADOW;
+    /* Deprecated types. To be removed in next version. */
     else if (StringsAreEqual(start, "grid_line", 9))
 	grid_type = XmGRID_LINE;
     else if (StringsAreEqual(start, "grid_shadow_in", 14))
 	grid_type = XmGRID_SHADOW_IN;
     else if (StringsAreEqual(start, "grid_shadow_out", 15))
 	grid_type = XmGRID_SHADOW_OUT;
-    else if (StringsAreEqual(start, "grid_row_shadow", 15))
-	grid_type = XmGRID_ROW_SHADOW;
-    else if (StringsAreEqual(start, "grid_column_shadow", 15))
-	grid_type = XmGRID_COLUMN_SHADOW;
     else {
 	XtDisplayStringConversionWarning(dpy, from->addr, XmRGridType);
 	return False;
     }
 
+    /* Deprecated types. To be removed in next version. */
+    if (grid_type >= XmGRID_LINE)
+        XtAppWarningMsg(
+	    XtDisplayToApplicationContext(dpy),
+	    "cvtStringToGridType", "deprecatedType",
+	    "XbaeMatrix",
+	    "Value for GridType is deprecated and will be removed in next release",
+	    NULL, NULL);
+
     /*
      * Store our return value
      */
@@ -960,7 +1057,13 @@ XtPointer *data;
 
 /* ARGSUSED */
 Boolean
+#ifdef __VMS
+/* According to Barry Stone VMS only allows function names with a
+   maximum length of 31 characters */
+CvtStringToMatrixScrollBarDisp(dpy, args, num_args, from, to, data)
+#else
 CvtStringToMatrixScrollBarDisplayPolicy(dpy, args, num_args, from, to, data)
+#endif
 Display *dpy;
 XrmValuePtr args;
 Cardinal *num_args;
